from fastapi import (
    APIRouter,
    Body,
    Depends,
    Form,
    HTTPException,
    Request,
    Response,
    status,
)

from fastapi_jwt_auth import AuthJWT

from sqlalchemy.ext.asyncio import AsyncConnection

from api.db.connection.session import get_connection_dep

from api.db.logic.account import get_user_login
from api.db.logic.keyring import get_key_id,create_key,update_key_id


from api.schemas.account.account import Role,Status
from api.schemas.endpoints.account_keyring import AccountKeyringUpdate

from api.schemas.account.account_keyring import AccountKeyring

from api.services.access_token_validadtion import AccessTokenValidadtion
from api.services.user_role_validation import db_user_role_validation
from api.services.update_data_validation import update_key_data_changes


api_router = APIRouter(
    prefix="/keyring",
    tags=["User KeyringModel"],
)


@api_router.get("/{user_id}/{key_id}")
async def get_keyring(
                    key_id: str,
                     connection: AsyncConnection = Depends(get_connection_dep),
                     Authorize: AuthJWT = Depends()):

    current_user = AccessTokenValidadtion(Authorize)

    authorize_user = await db_user_role_validation(connection, current_user)

    keyring = await get_key_id(connection, key_id)

    if keyring is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Key not found")

    return keyring


@api_router.post("/{user_id}/{key_id}")
async def create_keyring(
    user_id: int,
    key_id: str,
    key: AccountKeyringUpdate,
    connection: AsyncConnection = Depends(get_connection_dep),
    Authorize: AuthJWT = Depends()
):

    current_user = AccessTokenValidadtion(Authorize)

    authorize_user = await db_user_role_validation(connection, current_user)

    keyring = await get_key_id(connection, key_id)

    if keyring is None:
        print(key.key_type)
        user_new = await create_key(connection,key, key_id, )
        return user_new

    else:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="An keyring with this information already exists.")



@api_router.put("/{user_id}/{key_id}")
async def update_keyring(
    user_id: int,
    key_id: str,
    keyring_update: AccountKeyringUpdate,
    connection: AsyncConnection = Depends(get_connection_dep),
    Authorize: AuthJWT = Depends()
):
    current_user = AccessTokenValidadtion(Authorize)

    authorize_user = await db_user_role_validation(connection, current_user)


    keyring = await get_key_id(connection, key_id)
    if keyring is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="keyring not found")


    update_values = update_key_data_changes(keyring_update,keyring)

    if update_values is None:
        return keyring


    await update_key_id(connection, update_values, keyring)


    keyring = await get_key_id(connection, key_id)

    return keyring

@api_router.delete("/{user_id}/{key_id}")
async def delete_keyring(
    user_id: int,
    key_id: str,
    connection: AsyncConnection = Depends(get_connection_dep),
    Authorize: AuthJWT = Depends()
):
    current_user = AccessTokenValidadtion(Authorize)

    authorize_user = await db_user_role_validation(connection, current_user)


    keyring = await get_key_id(connection, key_id)
    if keyring is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="keyring not found")


    keyring_update = AccountKeyringUpdate(status=Status.DELETED.value)

    update_values = update_key_data_changes(keyring_update,keyring)

    if update_values is None:
        return keyring

    await update_key_id(connection, update_values, keyring)


    keyring = await get_key_id(connection, key_id)

    return keyring
